@@ -12,6 +12,7 @@ Every Config.verification_update_frequency, the verifications will be reprocesse
 """
 import asyncio
 import logging
+import re
 import time
 import rapidjson
 from datetime import datetime, timedelta
@@ -64,16 +65,16 @@ class Verifier(Periodic):
         except Exception:
             self.log.exception("bad metadata")
         else:
-            identifier = meta.get('nip05', '')
+            identifier = meta.get('nip05', '').lower().strip()
             self.log.debug("Found identifier %s in event %s", identifier, event)
             if '@' in identifier:
                 # queue this identifier as a candidate
-                domain = identifier.split('@', 1)[1].lower()
-                if self.check_allowed_domains(domain):
+                uname, domain = identifier.split('@', 1)
+                if self.check_allowed_domains(domain) and re.fullmatch("[a-z0-9\\._-]+", uname):
                     await self.queue.put([None, identifier, 0, event.pubkey, event.id_bytes])
                     return True
                 else:
-                    self.log.error("Illegal domain in identifier %s", identifier)
+                    self.log.error("Illegal identifier %s", identifier)
         return False
 
     def check_allowed_domains(self, domain):
@@ -170,8 +171,8 @@ class Verifier(Periodic):
             success, failure = await self.process_verifications(candidates)
         except Exception:
             self.log.exception("process_verifications")
-        else:
-            if success or failure:
+        if success or failure:
+            try:
                 async with self.db.begin() as conn:
                     for vid, identifier, metadata_id in success:
                         if vid is None:
@@ -185,7 +186,9 @@ class Verifier(Periodic):
                             continue
                         else:
                             await conn.execute(sa.update(Verification).where(Verification.c.id == vid).values({'failed_at': datetime.utcnow()}))
-                self.log.info("Saved success:%d failure:%d", len(success), len(failure))
+            except Exception:
+                self.log.exception("saving verifications")
+            self.log.info("Saved success:%d failure:%d", len(success), len(failure))
         self._last_run = time.time()
 
     def get_aiohttp_session(self):
@@ -201,11 +204,11 @@ class Verifier(Periodic):
                     pubkey = pubkey.hex()
                 self.log.info("Checking verification for %s. Last verified %s", identifier, verified_at)
                 uname, domain = identifier.split('@', 1)
-                domain = domain.lower()
                 if not self.check_allowed_domains(domain):
                     # how did this record get here?
                     self.log.warning("skipping verification for disallowed domain %s", identifier)
                     continue
+
                 # request well-known url
                 url = f'https://{domain}/.well-known/nostr.json?name={uname}'
                 self.log.info("Requesting %s", url)
