@@ -2,6 +2,7 @@ import logging, logging.config
 import unittest
 import os.path
 import time
+import sqlalchemy
 
 from nostr_relay.config import Config
 Config.load(os.path.join(os.path.dirname(__file__), './test_config.yaml'), reload=True)
@@ -23,14 +24,17 @@ class BaseTestsWithStorage(unittest.IsolatedAsyncioTestCase):
 
     async def asyncSetUp(self):
         from nostr_relay.storage import get_storage, get_metadata
-        self.storage = get_storage(reload=True)
+        self.storage = get_storage()
         await self.storage.setup()
         async with self.storage.db.begin() as conn:
             await conn.run_sync(get_metadata().create_all)
 
     async def asyncTearDown(self):
         async with self.storage.db.begin() as conn:
-            await conn.execute(self.storage.EventTable.delete())
+            try:
+                await conn.execute(self.storage.EventTable.delete())
+            except sqlalchemy.exc.OperationalError:
+                pass
 
     def make_event(self, privkey, pubkey=None, kind=0, created_at=None, tags=None, as_dict=True, **kwargs):
         if pubkey is None:
