@@ -19,7 +19,7 @@ class AuthTests(BaseTestsWithStorage):
         assert auth.actions == {'save': set('a'), 'query': set('a')}
 
     async def test_can_perform(self):
-        async with get_storage(reload=True) as storage:
+        async with get_storage() as storage:
             auth = Authenticator(storage, {'enabled': True, 'actions': {Action.save: Role.writer, Action.query: Role.reader}})
 
             token = {
@@ -84,6 +84,14 @@ class AuthTests(BaseTestsWithStorage):
 
         assert e.exception.args[0] == 'invalid: Too old'
 
+        too_new = Event(kind=22242, pubkey=pubkey1, created_at=time.time() + 605, tags=[('relay', 'ws://localhost:6969'), ('challenge', challenge)])
+        too_new.sign(privkey1)
+
+        with self.assertRaises(AuthenticationError) as e:
+            await auth.authenticate(too_new.to_json_object(), challenge)
+
+        assert e.exception.args[0] == 'invalid: Too new'
+
         good_event = Event(kind=22242, pubkey=pubkey1, created_at=time.time(), tags=[('relay', 'ws://localhost:6969'), ('challenge', challenge)])
         good_event.sign(privkey1)
 
@@ -97,3 +105,22 @@ class AuthTests(BaseTestsWithStorage):
         token = await auth.authenticate(good_event.to_json_object(), challenge)
         assert token['pubkey'] == pubkey1
         assert token['roles'] == set('rw')
+
+    async def test_roles(self):
+        pubkey1 = '5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d'
+        pubkey2 = '5de724fcabfb5ffd14e48a18f329092f345d0d5ed9f0f02903f40ec02753b011'
+
+        await self.storage.authenticator.set_roles(pubkey1, 'rw')
+        assert await self.storage.authenticator.get_roles(pubkey1) == set(['r', 'w'])
+
+        assert await self.storage.authenticator.get_roles(pubkey2) == set(['a'])
+        await self.storage.authenticator.set_roles(pubkey2, 's')
+        assert await self.storage.authenticator.get_roles(pubkey2) == set(['s'])
+
+        all_roles = []
+        async for pubkey, roles in self.storage.authenticator.get_all_roles():
+            all_roles.append((pubkey, roles))
+        assert all_roles == [(pubkey1, set(['r', 'w'])), (pubkey2, set('s'))]
+
+        await self.storage.authenticator.set_roles(pubkey1, 's')
+
